import os
import time
from argparse import Namespace

import pytest

from arkindex.mock import MockApiClient
from arkindex_worker.worker.base import BaseWorker
from worker_thumbnails_generator.worker import ThumbnailsGenerator


@pytest.fixture(autouse=True)
def _disable_sleep(monkeypatch):
    """
    Do not sleep at all in between API executions
    when errors occur in unit tests.
    This speeds up the test execution a lot
    """
    monkeypatch.setattr(time, "sleep", lambda x: None)


@pytest.fixture(autouse=True)
def _setup_environment(responses, monkeypatch) -> None:
    """Setup needed environment variables"""

    # Allow accessing remote API schemas
    # defaulting to the prod environment
    schema_url = os.environ.get(
        "ARKINDEX_API_SCHEMA_URL",
        "https://demo.arkindex.org/api/v1/openapi/?format=openapi-json",
    )
    responses.add_passthru(schema_url)

    # Set schema url in environment
    os.environ["ARKINDEX_API_SCHEMA_URL"] = schema_url
    # Setup a fake worker run ID
    os.environ["ARKINDEX_WORKER_RUN_ID"] = "1234-thumbnails-generator"
    # Setup a fake corpus ID
    os.environ["ARKINDEX_CORPUS_ID"] = "1234-corpus-id"

    # Setup a mock api client instead of using a real one
    def mock_setup_api_client(self):
        self.api_client = MockApiClient()

    monkeypatch.setattr(BaseWorker, "setup_api_client", mock_setup_api_client)


@pytest.fixture()
def mock_worker() -> ThumbnailsGenerator:
    worker = ThumbnailsGenerator()
    worker.args = Namespace(dev=False)
    worker.api_client = MockApiClient()

    # Default configuration
    worker.thumbnail_width = 900
    worker.thumbnail_height = 400
    worker.first_n = 3
    worker.corpus_types = {
        "folder": {"id": "folder_type_id", "slug": "folder", "folder": True},
        "page": {"id": "page_type_id", "slug": "page", "folder": False},
    }

    return worker


@pytest.fixture()
def page_1_payload() -> dict:
    return {
        "id": "page_1",
        "type": "page",
        "name": "1",
        "zone": {
            "polygon": [[0, 0], [1000, 0], [1000, 500], [0, 500], [0, 0]],
            "image": {
                "width": 1000,
                "height": 500,
                "url": "https://iiifserver/image1",
                "s3_url": None,
                "server": {
                    "url": "https://iiifserver",
                    "max_width": None,
                    "max_height": None,
                },
            },
            "url": "https://iiifserver/image1",
        },
        "rotation_angle": 0,
        "mirrored": False,
    }


@pytest.fixture()
def page_2_payload() -> dict:
    return {
        "id": "page_2",
        "type": "page",
        "name": "2",
        "zone": {
            "polygon": [[0, 0], [1000, 0], [1000, 500], [0, 500], [0, 0]],
            "image": {
                "width": 1000,
                "height": 500,
                "url": "https://iiifserver/image2",
                "s3_url": None,
                "server": {
                    "url": "https://iiifserver",
                    "max_width": None,
                    "max_height": None,
                },
            },
            "url": "https://iiifserver/image2",
        },
        "rotation_angle": 0,
        "mirrored": False,
    }


@pytest.fixture()
def page_3_payload() -> dict:
    return {
        "id": "page_3",
        "type": "page",
        "name": "3",
        "zone": {
            "polygon": [[0, 0], [1000, 0], [1000, 500], [0, 500], [0, 0]],
            "image": {
                "width": 1000,
                "height": 500,
                "url": "https://iiifserver/image3",
                "s3_url": None,
                "server": {
                    "url": "https://iiifserver",
                    "max_width": None,
                    "max_height": None,
                },
            },
            "url": "https://iiifserver/image3",
        },
        "rotation_angle": 0,
        "mirrored": False,
    }


@pytest.fixture()
def pages_payload(
    page_1_payload: dict, page_2_payload: dict, page_3_payload: dict
) -> list[dict]:
    return [page_1_payload, page_2_payload, page_3_payload]
